/*
 Ariadna-on-Line DTMF decoder.
 Строки 15-16 - Для расширенного варианта ДТМФ
 Строки 22-23 - Специфика Гёрцеля. Например для входной 1000 Гц, частота опознавания - 910 Гц. И тд. 
 Строка 25-26 - Частоты измерения шума. Для адаптивного порога шума.
 Строка 64-65 - Фиксированный порог шума. Эмпирически выбран.
 Строка 87-88 - Степень превышения сигнала над шумом. Выставлено эмпирически.  
*/
//==================================
#include <Goertzel.h>            // include the Goertzel library
#define ADC_MIDPOINT 512         // Define the ADC midpoint (512 is the default)
#define SAMPLE_RATE 8800         // Define the sample rate (8800 S/s is the default for 16 MHz Boards)
#define numSampl 100             // Define the samples quantity
#define inputADC A0              // Define signal input ADC channel

int Samples[numSampl];           // Array buffer for ADC samples
int X;                           // Coordinate of latter
int Y;                           // Coordinate of Letter

//const float FreqL[4] = {586.0, 635.0, 697.0, 770.0, 852.0, 941.0};         // Expanded DTMF Low Freqs
//const float FreqH[4] = {1209.0, 1336.0, 1477.0, 1633.0, 1805.0, 1994.0};   // Expanded DTMF High Freq 

const float FreqL[4] = {635, 697, 770, 852};                 // DTMF Geterodin LF
const float FreqH[4] = {1095, 1209, 1336, 1477};             // DTMF Geterodin HF

//const float BazeLF = 500;   // Bazement LF messurement Freq (500 HZ) 
//const float BazeHF = 1000;  // Bazement HF messurement Freq (1000 HZ)  
 
float PrevMagL;
float PrevMagH;
float CurrMagL[4];
float CurrMagH[4];

const char Letter[4][4] = {{'1', '2', '3', 'A'},{'4', '5', '6', 'B'},{'7', '8', '9', 'C'},{'*', '0', '#', 'D'}};

boolean Goertz = false;       // Status class instantiation

//  Goertzel TONE_LF(BazeLF, SAMPLE_RATE);   // Измерение уровня шума НЧ
//  Goertzel TONE_HF(BazeHF,SAMPLE_RATE);    // Измерение уровня шума ВЧ
   
  Goertzel TONE0(FreqL[0],SAMPLE_RATE);
  Goertzel TONE1(FreqL[1],SAMPLE_RATE);
  Goertzel TONE2(FreqL[2],SAMPLE_RATE);
  Goertzel TONE3(FreqL[3],SAMPLE_RATE);

  Goertzel TONE4(FreqH[0],SAMPLE_RATE);
  Goertzel TONE5(FreqH[1],SAMPLE_RATE);
  Goertzel TONE6(FreqH[2],SAMPLE_RATE);
  Goertzel TONE7(FreqH[3],SAMPLE_RATE);
  
//==================================
void setup() {

  Serial.begin(9600);
  Serial.println(F("Wait for DTMF"));   // Инфо
  
  pinMode(inputADC, INPUT);
  analogReference(DEFAULT);
  
  Goertz = true;  // Процесс Герцеля включен
}
//==================================
void loop() {
  
  PrevMagL = 200;     // Minimal signal level. Empiricly selected
  PrevMagH = 200;     // Minimal signal level. Empiricly selected
  
  X = 4;
  Y = 4;

  for (int n = 0; n < numSampl; n++)  {  // Get 100 ADC samples from A0 and store them in the buffer
  Samples[n] = analogRead(inputADC);
  }

//  PrevMagL    = TONE_LF.Mag(Samples,numSampl);     //
//  PrevMagH    = TONE_HF.Mag(Samples,numSampl);     //

  CurrMagL[0] = TONE0.Mag(Samples, numSampl);
  CurrMagL[1] = TONE1.Mag(Samples, numSampl);
  CurrMagL[2] = TONE2.Mag(Samples, numSampl);
  CurrMagL[3] = TONE3.Mag(Samples, numSampl);
  CurrMagH[0] = TONE4.Mag(Samples, numSampl);
  CurrMagH[1] = TONE5.Mag(Samples, numSampl);
  CurrMagH[2] = TONE6.Mag(Samples, numSampl);
  CurrMagH[3] = TONE7.Mag(Samples, numSampl);

  PrevMagL = PrevMagL * 1.3;   // Set what you want
  PrevMagH = PrevMagH * 1.3;   // Set what you want
  
  for (int i = 0; i < 4; i++) {
   if (CurrMagL[i] > PrevMagL) {
      X = i;
      PrevMagL = CurrMagL[i];
      }
   if (CurrMagH[i] > PrevMagH) {
      Y = i;
      PrevMagH = CurrMagH[i];
      }
   }
   
   if ((X < 4) && (Y < 4)) { 
   Serial.println(Letter[X][Y]);
   Serial.println(" - -");
   }
   delay(1000);   
}
